@@ -186,9 +186,9 @@ pi_file_open (char *name)
   memcpy(ip->name, p, 32);
   ip->flags = get_short(p+32);
   ip->version = get_short(p+34);
-  ip->crdate = pilot_time_to_unix_time (get_long(p+36));
-  ip->moddate = pilot_time_to_unix_time (get_long(p+40));
-  ip->backupdate = pilot_time_to_unix_time (get_long(p+44));
+  ip->createDate = pilot_time_to_unix_time (get_long(p+36));
+  ip->modifyDate = pilot_time_to_unix_time (get_long(p+40));
+  ip->backupDate = pilot_time_to_unix_time (get_long(p+44));
   ip->modnum = get_long(p+48);
   app_info_offset = get_long(p+52);
   sort_info_offset = get_long(p+56);
@@ -202,9 +202,9 @@ pi_file_open (char *name)
 
 #ifdef DEBUG  
   printf("pi_file_open:\n Name '%s', flags 0x%4.4X, version %d\n", ip->name, ip->flags, ip->version);
-  printf(" Creation date %s\n", ctime(&ip->crdate));
-  printf(" Modification date %s\n", ctime(&ip->moddate));
-  printf(" Backup date %s\n", ctime(&ip->backupdate));
+  printf(" Creation date %s\n", ctime(&ip->createDate));
+  printf(" Modification date %s\n", ctime(&ip->modifyDate));
+  printf(" Backup date %s\n", ctime(&ip->backupDate));
   printf(" Appinfo size %d, sortinfo size %d\n", pf->app_info_size, pf->sort_info_size);
   printf(" Type '%s'", printlong(ip->type));
   printf(" Creator '%s', seed 0x%8.8lX\n", printlong(ip->creator), pf->unique_id_seed);
@@ -386,10 +386,10 @@ pi_file_set_rbuf_size (struct pi_file *pf, int size)
 
   if (size > pf->rbuf_size) {
     if (pf->rbuf_size == 0) {
-      new_size = 64*1024;
+      new_size = size+2048;
       rbuf = malloc (new_size);
     } else {
-      new_size = size;
+      new_size = size+2048;
       rbuf = realloc (pf->rbuf, new_size);
     }
 
@@ -424,15 +424,15 @@ pi_file_read_resource (struct pi_file *pf, int idx,
 
   entp = &pf->entries[idx];
 
-  if (pi_file_set_rbuf_size (pf, entp->size) < 0)
-    return (-1);
-
-  fseek (pf->f, pf->entries[idx].offset, SEEK_SET);
-  if (fread (pf->rbuf, 1, entp->size, pf->f) != (size_t)entp->size)
-    return (-1);
-
-  if (bufp)
+  if (bufp) {
+    if (pi_file_set_rbuf_size (pf, entp->size) < 0)
+      return (-1);
+    fseek (pf->f, pf->entries[idx].offset, SEEK_SET);
+    if (fread (pf->rbuf, 1, entp->size, pf->f) != (size_t)entp->size)
+      return (-1);
     *bufp = pf->rbuf;
+  }
+  
   if (sizep)
     *sizep = entp->size;
   if (type)
@@ -453,6 +453,10 @@ pi_file_read_record (struct pi_file *pf, int idx,
 
   if (pf->for_writing)
     return (-1);
+    
+#ifdef DEBUG
+  fprintf(stderr, "Reading record %d\n", idx);
+#endif  
 
   ip = &pf->info;
 
@@ -463,16 +467,21 @@ pi_file_read_record (struct pi_file *pf, int idx,
     return (-1);
 
   entp = &pf->entries[idx];
+  
+#ifdef DEBUG
+  fprintf(stderr, "record size is %d\n", entp->size);
+#endif  
 
-  if (pi_file_set_rbuf_size (pf, entp->size) < 0)
-    return (-1);
 
-  fseek (pf->f, pf->entries[idx].offset, SEEK_SET);
-  if (fread (pf->rbuf, 1, entp->size, pf->f) != (size_t)entp->size)
-    return (-1);
-
-  if (bufp)
+  if (bufp) {
+    if (pi_file_set_rbuf_size (pf, entp->size) < 0)
+      return (-1);
+    fseek (pf->f, pf->entries[idx].offset, SEEK_SET);
+    if (fread (pf->rbuf, 1, entp->size, pf->f) != (size_t)entp->size)
+      return (-1);
     *bufp = pf->rbuf;
+  }
+  
   if (sizep)
     *sizep = entp->size;
   if (attrp)
@@ -734,9 +743,9 @@ pi_file_close_for_write (struct pi_file *pf)
   memcpy(p, ip->name, 32);
   set_short(p+32, ip->flags);
   set_short(p+34, ip->version);
-  set_long(p+36, unix_time_to_pilot_time(ip->crdate));
-  set_long(p+40, unix_time_to_pilot_time(ip->moddate));
-  set_long(p+44, unix_time_to_pilot_time(ip->backupdate));
+  set_long(p+36, unix_time_to_pilot_time(ip->createDate));
+  set_long(p+40, unix_time_to_pilot_time(ip->modifyDate));
+  set_long(p+44, unix_time_to_pilot_time(ip->backupDate));
   set_long(p+48, ip->modnum);
   set_long(p+52, pf->app_info_size ? offset : 0); offset += pf->app_info_size;
   set_long(p+56, pf->sort_info_size ? offset : 0); offset += pf->sort_info_size;
@@ -824,7 +833,7 @@ int pi_file_retrieve(struct pi_file * pf, int socket, int cardno)
     }
   else
     for(j=0;j<l;j++) {
-  	  unsigned long id;
+	  unsigned long id;
   	  int size;
   	  int attr;
   	  int category;
@@ -912,7 +921,15 @@ int pi_file_install(struct pi_file * pf, int socket, int cardno)
     free(buffer);
    
   /* Resource or record? */
-  if(pf->info.flags & dlpDBFlagResource)
+  if(pf->info.flags & dlpDBFlagResource) {
+    for(j=0;j<pf->nentries;j++) {
+      int size;
+      if( (pi_file_read_resource(pf, j, 0, &size, 0, 0)==0) &&
+          (size > 65536)) {
+	fprintf(stderr, "Database contains resource over 64K!\n");
+        goto fail;
+      }
+    }
     for(j=0;j<pf->nentries;j++) {
       unsigned long type;
       int id;
@@ -926,7 +943,15 @@ int pi_file_install(struct pi_file * pf, int socket, int cardno)
       if (type == pi_mktag ('b','o','o','t')) 
         reset = 1;
     }
-  else
+  } else {
+    for(j=0;j<pf->nentries;j++) {
+      int size;
+      if( ((pi_file_read_record(pf, j, 0, &size, 0, 0, 0)==0)) &&
+          (size > 65536)) {
+	fprintf(stderr, "Database contains record over 64K!\n");
+        goto fail;
+      }
+    }
     for(j=0;j<pf->nentries;j++) {
       unsigned long id;
       int size;
@@ -940,6 +965,7 @@ int pi_file_install(struct pi_file * pf, int socket, int cardno)
       if(dlp_WriteRecord(socket, db, attr, id, category, buffer, size, 0)<0)
         goto fail;
     }
+  }
   
   if(reset)
     dlp_ResetSystem(socket);
@@ -973,7 +999,15 @@ int pi_file_merge(struct pi_file * pf, int socket, int cardno)
     reset = 1;
       
   /* Resource or record? */
-  if(pf->info.flags & dlpDBFlagResource)
+  if(pf->info.flags & dlpDBFlagResource) {
+    for(j=0;j<pf->nentries;j++) {
+      int size;
+      if( (pi_file_read_resource(pf, j, 0, &size, 0, 0)==0) &&
+          (size > 65536)) {
+	fprintf(stderr, "Database contains resource over 64K!\n");
+        goto fail;
+      }
+    }
     for(j=0;j<pf->nentries;j++) {
       unsigned long type;
       int id;
@@ -987,7 +1021,15 @@ int pi_file_merge(struct pi_file * pf, int socket, int cardno)
       if (type == pi_mktag ('b','o','o','t')) 
         reset = 1;
     }
-  else
+  } else {
+    for(j=0;j<pf->nentries;j++) {
+      int size;
+      if( ((pi_file_read_record(pf, j, 0, &size, 0, 0, 0)==0)) &&
+          (size > 65536)) {
+	fprintf(stderr, "Database contains record over 64K!\n");
+        goto fail;
+      }
+    }
     for(j=0;j<pf->nentries;j++) {
       unsigned long id;
       int size;
@@ -1001,6 +1043,7 @@ int pi_file_merge(struct pi_file * pf, int socket, int cardno)
       if(dlp_WriteRecord(socket, db, attr, 0, category, buffer, size, 0)<0)
         goto fail;
     }
+  }
   
   if(reset)
     dlp_ResetSystem(socket);
