@@ -4,6 +4,10 @@
  *
  * This is free software, licensed under the GNU Public License V2.
  * See the file COPYING for details.
+ *
+ * Modifications by Diego Zamboni to allow it to read mail from an MH-style
+ * mailbox into the Pilot.
+ *
  */
 
 #define PILOTPORT	""
@@ -14,6 +18,7 @@
 #define SENDMAIL	"/usr/lib/sendmail -t -i"
 #define POPKEEP 	"keep"
 #define DISPOSE		"keep"
+#define TOPILOT_MHDIR   ""
 
 /* Todo: truncation, filtering, priority, notification */
 
@@ -35,6 +40,22 @@
 
 extern time_t parsedate(char * p);
 
+void markline(char *msg) {
+    while((*msg)!='\n' && (*msg)!=0 ) {
+	msg++;
+    }
+    (*msg)=0;
+}
+
+int openmhmsg(char *dir, int num) {
+    char filename[1000];
+
+
+    sprintf(filename, "%s/%d", dir, num);
+
+    return open(filename, O_RDONLY);
+}
+
 /* Fold into RFC822 style*/
 void fprintfold(FILE * f, char * t)
 {
@@ -158,9 +179,9 @@ void header(struct Mail * m, char * t)
 }
 
 
-void Help(char *argv[])
+void Help(char * progname)
 {
-  fprintf(stderr,"usage:%s [options]\n",argv[0]);
+  fprintf(stderr,"usage: %s [options]\n",progname);
   fprintf(stderr,"\n      <-p port> Serial port [PILOTPORT]");
   fprintf(stderr,"\n                   (defaults to '%s')", PILOTPORT);
   fprintf(stderr,"\n      <-h host> POP3 host (if empty, mail won't be received) [POPHOST]");
@@ -177,6 +198,8 @@ void Help(char *argv[])
   fprintf(stderr,"\n                   (defaults to '%s')", POPKEEP);
   fprintf(stderr,"\n      <-d keep|delete|file> Disposition of sent mail [PILOTDISPOSE]");
   fprintf(stderr,"\n                   (defaults to '%s')", DISPOSE);
+  fprintf(stderr,"\n      <-m mhdir> MH directory to download to Pilot [TOPILOT_MHDIR]");
+  fprintf(stderr,"\n                   (defaults to '%s')", TOPILOT_MHDIR);
   fprintf(stderr, "\n\nAll options may be specified via the environment variable named in brackets.\n");
   exit(0);
 }
@@ -197,11 +220,13 @@ int main(int argc, char *argv[])
   int ret;
   unsigned char buffer[0xffff];
   struct MailAppInfo tai;
-  struct MailPrefs p;
+  struct MailSyncPref p;
+  struct MailSignaturePref sig;
   struct sockaddr_in pop_addr;
   int popfd;
   struct hostent * hostent;
   int c;
+  char * progname = argv[0];
   
   char * from_address = 
 #ifdef PILOTFROM
@@ -251,6 +276,12 @@ int main(int argc, char *argv[])
 #else
   "";
 #endif
+  char * mh_dir =
+#ifdef TOPILOT_MHDIR
+  TOPILOT_MHDIR;
+#else
+  "";
+#endif
 
   extern char* optarg;
   extern int optind;
@@ -272,12 +303,12 @@ int main(int argc, char *argv[])
     pop_pass = getenv("POPPASS");
   if (getenv("PILOTPORT"))
     port = getenv("PILOTPORT");
-    
+  if (getenv("TOPILOT_MHDIR"))
+    mh_dir = getenv("TOPILOT_MHDIR");
+     
   signal(SIGINT, sigint);
 
-  optind = 0;
-  
-  while ((c = getopt(argc, argv, "s:p:d:f:h:u:p:h:P:k:")) != EOF) {
+  while ((c = getopt(argc, argv, "s:p:d:f:h:u:p:h:P:k:m:")) != EOF) {
     switch (c) {
       case 's':
         sendmail = optarg;
@@ -303,8 +334,11 @@ int main(int argc, char *argv[])
       case 'k':
         pop_keep = optarg;
         break;
+      case 'm':
+	mh_dir = optarg;
+	break;
       case 'H': case '?': default:
-        Help(argv);
+        Help(progname);
     }
   }
   argc -= optind;
@@ -312,26 +346,26 @@ int main(int argc, char *argv[])
   
   if (!strlen(port)) {
     fprintf(stderr, "Port must be set.\n\n");
-    Help(argv);
+    Help(progname);
   }
   
   
   if (!strlen(pop_host) && !strlen(sendmail)) {
     fprintf(stderr, "At least one of -h or -s must be set.\n\n");
-    Help(argv);
+    Help(progname);
   }
   
   if (strlen(pop_host)) {
     if (!strlen(pop_user)) {
       fprintf(stderr, "-u must be set to receive mail.\n\n");
-      Help(argv);
+      Help(progname);
     } else if (!strlen(pop_keep) || (strcmp(pop_keep, "keep") && strcmp(pop_keep,"delete"))) {
       fprintf(stderr, "-k must have an argument of 'keep' or 'delete'.\n\n");
-      Help(argv);
+      Help(progname);
     } 
   }
 
-  if ( !strlen( pop_pass ) )
+  if ( !strlen( pop_pass ) && strlen( pop_host ) )
     {
       pop_pass = getpass( "POP password: " );
     }
@@ -341,7 +375,7 @@ int main(int argc, char *argv[])
                              strcmp(dispose,"delete") &&
                              strcmp(dispose,"file"))) {
       fprintf(stderr, "-d must have an argument of 'keep', 'delete', or 'file'.\n\n");
-      Help(argv);
+      Help(progname);
     }
   }
   
@@ -374,8 +408,7 @@ int main(int argc, char *argv[])
   }
   
   memset(&tai, '\0', sizeof(struct MailAppInfo));
-  memset(&p, '\0', sizeof(struct MailPrefs));
-  tai.signature = 0;
+  memset(&p, '\0', sizeof(struct MailSyncPref));
 
   /* Ask the pilot who it is. */
   dlp_ReadUserInfo(sd,&U);
@@ -391,45 +424,42 @@ int main(int argc, char *argv[])
   }
   
   dlp_ReadAppBlock(sd, db, 0, buffer, 0xffff);
-  unpack_MailAppInfo(&tai, buffer, 0);
+  unpack_MailAppInfo(&tai, buffer, 0xffff);
   
   setbuf(stderr, 0);
   
-  p.synctype = 0;
-  p.gethigh = 0;
-  p.getcontaining = 0;
+  p.syncType = 0;
+  p.getHigh = 0;
+  p.getContaining = 0;
   p.truncate = 8*1024;
-  p.filterto = 0;
-  p.filterfrom = 0;
-  p.filtersubject = 0;
+  p.filterTo = 0;
+  p.filterFrom = 0;
+  p.filterSubject = 0;
   
   if (pi_version(sd) > 0x0100) {
     if (dlp_ReadAppPreference(sd, makelong("mail"), 1, 1, 0xffff, buffer, 0, 0)>=0) {
       printf("Got local backup mail preferences\n"); /* 2 for remote prefs */
-      unpack_MailPrefs(&p, buffer, 0);
+      unpack_MailSyncPref(&p, buffer, 0xffff);
     } else {
       printf("Unable to get mail preferences, trying current\n");
       if (dlp_ReadAppPreference(sd, makelong("mail"), 1, 1, 0xffff, buffer, 0, 0)>=0) {
         printf("Got local current mail preferences\n"); /* 2 for remote prefs */
-        unpack_MailPrefs(&p, buffer, 0);
+        unpack_MailSyncPref(&p, buffer, 0xffff);
       } else
         printf("Couldn't get any mail preferences.\n");
     }
     
-    if (tai.signature)
-      free(tai.signature);
-      
     if (dlp_ReadAppPreference(sd, makelong("mail"), 3, 1, 0xffff, buffer, 0, 0)>0) {
-      tai.signature=strdup(buffer);
+      unpack_MailSignaturePref(&sig, buffer, 0xffff);
     }
 
   }
 
   printf("Local Prefs: Sync=%d, High=%d, getc=%d, trunc=%d, to=|%s|, from=|%s|, subj=|%s|\n",
-     p.synctype, p.gethigh, p.getcontaining, p.truncate, p.filterto ? p.filterto : "<none>",
-     p.filterfrom ? p.filterfrom : "<none>", p.filtersubject ? p.filtersubject : "<none>");
+     p.syncType, p.getHigh, p.getContaining, p.truncate, p.filterTo ? p.filterTo : "<none>",
+     p.filterFrom ? p.filterFrom : "<none>", p.filterSubject ? p.filterSubject : "<none>");
 
-  printf("AppInfo Signature: |%s|\n\n", tai.signature ? tai.signature : "<None>");
+  printf("AppInfo Signature: |%s|\n\n", sig.signature ? sig.signature : "<None>");
 
 #if 0
   for (i=0;1;i++) {
@@ -525,15 +555,24 @@ int main(int argc, char *argv[])
 	  fprintfold(sendf,t.subject);
 	  fprintf(sendf,"\r\n");
 	}
+	fprintf(sendf, "X-mailer: pilot-mail-%d.%d.%d\r\n", PILOT_LINK_VERSION,PILOT_LINK_MAJOR,PILOT_LINK_MINOR);
 	fprintf(sendf,"\r\n"); /* Separate header */
 	
 	if (t.body) {
 	  fprintbody(sendf,t.body);
 	  fprintf(sendf,"\r\n");
 	}
-	if (t.signature && tai.signature) {
-	  fprintf(sendf,"\r\n-- \r\n");
-	  fprintbody(sendf,tai.signature);
+	if (t.signature && sig.signature) {
+	  char * c = sig.signature;
+	  while ((*c == '\r') || (*c == '\n'))
+	  	c++;
+	  if(
+	     strncmp(c,"--",2) &&
+	     strncmp(c,"__",2)
+	    ) {
+	    fprintf(sendf,"\r\n-- \r\n");
+	  }
+	  fprintbody(sendf,sig.signature);
 	  fprintf(sendf,"\r\n");
 	}
 
@@ -696,7 +735,7 @@ int main(int argc, char *argv[])
     
     t.body = strdup(buffer);
     
-    pack_Mail(&t, buffer, &len);
+    len = pack_Mail(&t, buffer, 0xffff);
     
     if (dlp_WriteRecord(sd, db, 0, 0, 0, buffer, len, 0)>0) {
       rec++;
@@ -729,8 +768,123 @@ endpop:
   
   }
 
+  if (strlen(mh_dir)) {
+
+      fprintf(stderr, "Reading directory %s... ", mh_dir);
+      fflush(stderr);
+
+  /* MH directory reading section */
+  
+  for(i=1;1;i++) {
+    int len;
+    char * msg;
+    int h;
+    struct Mail t;
+    int mhmsg;
+    
+    t.to = 0;
+    t.from = 0;
+    t.cc = 0;
+    t.bcc = 0;
+    t.subject = 0;
+    t.replyTo = 0;
+    t.sentTo = 0;
+    t.body = 0;
+    t.dated = 0;
+    
+    if((mhmsg=openmhmsg(mh_dir, i))<0) {
+	break;
+    }
+
+    fprintf(stderr, "%d ", i);
+    fflush(stderr);
+
+    /* Read the message */
+    len=0;
+    while((len<sizeof(buffer)) && 
+	  ((l=read(mhmsg, (char*)(buffer+len), (sizeof(buffer)-len)))>0)) {
+	len+=l;
+    }
+    buffer[len]=0;
+
+    if(l<0) {
+	perror("Error while reading message");
+	goto endmh;
+    }
+
+    msg = buffer;
+    h = 1;
+    while(h==1) {
+      markline(msg);
+      
+      if ((msg[0] == 0) && (msg[1] == 0)) {
+          break; /* End of message */
+      }
+      
+      if (msg[0] == 0) {
+          h = 0;
+          header(&t, 0);
+      } else 
+          header(&t, msg);
+      msg += strlen(msg)+1;
+    }
+
+    /* When we get here, we are done with the headers */
+    if((*msg)==0) {
+	/* Empty message */
+	h=1;
+    }
+
+    /* Well, we've now got the message. I bet _you_ feel happy with yourself. */
+    
+    if (h) {
+      /* Oops, incomplete message, still reading headers */
+      fprintf(stderr, "Incomplete message %d\n", i);
+      free_Mail(&t);
+      continue;
+    }
+
+    if (strlen(msg) > p.truncate) {
+	/* We could truncate it, but we won't for now */
+	fprintf(stderr, "Message %d too large (%ld bytes)\n", i, (long)strlen(msg));
+	free_Mail(&t);
+	continue;
+    }
+    
+    t.body = strdup(msg);
+    
+    len = pack_Mail(&t, buffer, 0xffff);
+    
+    if (dlp_WriteRecord(sd, db, 0, 0, 0, buffer, len, 0)>0) {
+      rec++;
+      if (strcmp(pop_keep,"delete")==0) { 
+	  char filename[1000];
+	  sprintf(filename, "%s/%d", mh_dir, i);
+	  close(mhmsg);
+	  if(unlink(filename)) {
+	      fprintf(stderr, "Error deleting message %d\n", i);
+	      dupe++;
+	  }
+	  continue;
+      } else
+        dupe++;
+    } else {
+      fprintf(stderr,"Error writing message to Pilot\n");
+    }
+    
+    free_Mail(&t);
+
+    close(mhmsg);
+  }
+  
+endmh:  
+  fprintf(stderr, "\n");
+
+  }
+
 end:  
-  free_MailPrefs(&p);
+  free_MailSyncPref(&p);
+  free_MailSignaturePref(&sig);
   free_MailAppInfo(&tai);
   
   dlp_ResetLastSyncPC(sd);
@@ -738,7 +892,7 @@ end:
   /* Close the database */
   dlp_CloseDB(sd, db);
 
-  sprintf(buffer, "Finished transferring mail. %d message%s sent, %d message%s receieved.\n",
+  sprintf(buffer, "Finished transferring mail. %d message%s sent, %d message%s received.\n",
     sent, (sent == 1) ? "" : "s", rec, (rec == 1) ? "" : "s");
   if (lost || dupe)
     sprintf(buffer+strlen(buffer), "(And %d lost, %d duplicated. Sorry.)\n", lost, dupe);
