@@ -22,6 +22,14 @@ char * tclquote(char * in)
   char * pos;
   int len;
   
+  /* Skip leading bullet (and any whitespace after) */
+  if (in[0] == '\x95') {
+    ++in;
+    while(in[0] == ' ' || in[0] == '\t') {
+	++in;
+    }
+  }
+
   len = 3;
   pos = in;
   while(*pos) {
@@ -49,6 +57,17 @@ char * tclquote(char * in)
   return buffer;
 }
 
+static void Usage(char *progname)
+{
+    fprintf(stderr,"Usage: %s [-d] [-p pubtext] [%s] calfile\n"
+	    "Note: calfile will be overwritten!\n"
+	    "   -d         : Datebook only, no Todos\n"
+	    "   -p pubtext : Replace text of items not starting with a bullet "
+	    "with pubtext\n",
+	progname, TTYPrompt);
+    exit(2);
+}
+
 int main(int argc, char *argv[])
 {
   struct pi_sockaddr addr;
@@ -61,10 +80,36 @@ int main(int argc, char *argv[])
   unsigned char buffer[0xffff];
   char cmd[128];
   struct ToDoAppInfo tai;
+  int read_todos = 1;
+  char *pubtext = NULL;
+  char *progname = argv[0];
+  char *port = getenv("PILOTPORT");
 
-  if (argc < 3) {
-    fprintf(stderr,"usage:%s %s calfile # Calfile will be overwritten!\n",argv[0],TTYPrompt);
-    exit(2);
+  if (!port) port = "/dev/pilot";
+
+  while (argc > 1 && argv[1][0] == '-') {
+    if (!strcmp(argv[1], "-d")) {
+	/* Datebook only */
+	read_todos = 0;
+    } else if (!strcmp(argv[1], "-p")) {
+	/* Public only, text supplied */
+	if (argv[2] == NULL) {
+	    Usage(progname);
+	}
+	pubtext = argv[2];
+	--argc; ++argv;
+    } else {
+	Usage(progname);
+    }
+    --argc; ++argv;
+  }
+
+  if (argc != 2 && argc != 3) {
+    Usage(progname);
+  }
+  if (argc == 3) {
+    port = argv[1];
+    --argc; ++argv;
   }
   if (!(sd = pi_socket(PI_AF_SLP, PI_SOCK_STREAM, PI_PF_PADP))) {
     perror("pi_socket");
@@ -72,7 +117,7 @@ int main(int argc, char *argv[])
   }
     
   addr.pi_family = PI_AF_SLP;
-  strcpy(addr.pi_device,argv[1]);
+  strcpy(addr.pi_device,port);
   
   ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
   if(ret == -1) {
@@ -98,6 +143,13 @@ int main(int argc, char *argv[])
   /* Tell user (via Pilot) that we are starting things up */
   dlp_OpenConduit(sd);
   
+  unlink(argv[1]);
+  sprintf(cmd, "ical -f - -calendar %s", argv[1]);
+  ical = popen(cmd, "w");
+  
+  fprintf(ical,"calendar cal $ical(calendar)\n");
+  
+  if (read_todos) {
   /* Open the ToDo database, store access handle in db */
   if(dlp_OpenDB(sd, 0, 0x80|0x40, "ToDoDB", &db) < 0) {
     puts("Unable to open ToDoDB");
@@ -106,13 +158,7 @@ int main(int argc, char *argv[])
   }
   
   dlp_ReadAppBlock(sd, db, 0, buffer, 0xffff);
-  unpack_ToDoAppInfo(&tai, buffer, 0);
-  
-  unlink(argv[2]);
-  sprintf(cmd, "ical -f - -calendar %s", argv[2]);
-  ical = popen(cmd, "w");
-  
-  fprintf(ical,"calendar cal $ical(calendar)\n");
+  unpack_ToDoAppInfo(&tai, buffer, 0xffff);
   
   for (i=0;1;i++) {
   	struct ToDo t;
@@ -131,12 +177,14 @@ int main(int argc, char *argv[])
 	unpack_ToDo(&t, buffer, len);
 	
 	fprintf(ical,"set n [notice]\n");
-	fprintf(ical,"$n text %s\n", tclquote(t.description));
+	/* '\x95' is the "bullet" character */
+	fprintf(ical,"$n text %s\n", tclquote((pubtext &&
+	    t.description[0] != '\x95') ? pubtext : t.description));
 	fprintf(ical,"$n date [date today]\n");
 	fprintf(ical,"$n todo 1\n");
 	fprintf(ical,"$n option Priority %d\n", t.priority);
         sprintf(id_buf, "%lx", id);
-        fprintf(ical,"$i option PilotRecordId %s\n", id_buf);
+        fprintf(ical,"$n option PilotRecordId %s\n", id_buf);
 	fprintf(ical,"$n done %d\n", t.complete ? 1 : 0);
 	fprintf(ical,"cal add $n\n");
 	
@@ -147,6 +195,7 @@ int main(int argc, char *argv[])
   dlp_CloseDB(sd, db);
 
   dlp_AddSyncLogEntry(sd, "Read todos from Pilot.\n");
+  }
 
   /* Open the Datebook's database, store access handle in db */
   if(dlp_OpenDB(sd, 0, 0x80|0x40, "DatebookDB", &db) < 0) {
@@ -187,26 +236,28 @@ int main(int argc, char *argv[])
 	  end   = a.end.tm_hour*60 + a.end.tm_min;
 
 	  fprintf(ical,"$i starttime %d\n", start);
-	  fprintf(ical,"$i length %d\n", end-start+1);
+	  fprintf(ical,"$i length %d\n", end-start);
 	}
 	
-	fprintf(ical,"$i text %s\n", tclquote(a.description));
+	/* '\x95' is the "bullet" character */
+	fprintf(ical,"$i text %s\n", tclquote((pubtext &&
+	    a.description[0] != '\x95') ? pubtext : a.description));
 	
 	fprintf(ical,"set begin [date make %d %d %d]\n", a.begin.tm_mday,a.begin.tm_mon+1,a.begin.tm_year+1900);
 	
-	if (a.repeatFreq) {
+	if (a.repeatFrequency) {
 	  if (a.repeatType == repeatDaily) {
-	    fprintf(ical,"$i dayrepeat %d $begin\n", a.repeatFreq);
+	    fprintf(ical,"$i dayrepeat %d $begin\n", a.repeatFrequency);
 	  } else if(a.repeatType == repeatMonthlyByDate) {
-	    fprintf(ical,"$i month_day %d $begin %d\n",a.begin.tm_mon+1,a.repeatFreq);
+	    fprintf(ical,"$i month_day %d $begin %d\n",a.begin.tm_mon+1,a.repeatFrequency);
 	  } else if(a.repeatType == repeatMonthlyByDay) {
-	    if (a.repeatOn>=domLastSun) {
-	      fprintf(ical,"$i month_last_week_day %d 1 $begin %d\n", a.repeatOn % 7 + 1,
-	                                                    a.repeatFreq);
+	    if (a.repeatDay>=domLastSun) {
+	      fprintf(ical,"$i month_last_week_day %d 1 $begin %d\n", a.repeatDay % 7 + 1,
+	                                                    a.repeatFrequency);
 	    } else {
-	      fprintf(ical,"$i month_week_day %d %d $begin %d\n", a.repeatOn % 7 + 1,
-	                                                    a.repeatOn / 7 + 1,
-	                                                    a.repeatFreq);
+	      fprintf(ical,"$i month_week_day %d %d $begin %d\n", a.repeatDay % 7 + 1,
+	                                                    a.repeatDay / 7 + 1,
+	                                                    a.repeatFrequency);
 	    }
 	  } else if(a.repeatType == repeatWeekly) {
 	    /*
@@ -215,36 +266,25 @@ int main(int argc, char *argv[])
 	     * do days of the week and a repeat-frequency > 1, so do the
 	     * best we can and go on.
 	     */
-	    if (a.repeatFreq > 1) {
+	    if (a.repeatFrequency > 1) {
 		int ii, found;
-		for (ii = 1, found = 0; ii < 128; ii <<= 1) {
-		    if (a.repeatOn & i)
+		for (ii = 0, found = 0; ii < 7; ii++) {
+		    if (a.repeatDays[i])
 			found++;
 		}
 		if (found > 1)
 		    fprintf(stderr, "Incomplete translation of %s\n",
 			    a.description);
-		fprintf(ical,"$i dayrepeat %d $begin\n", a.repeatFreq * 7);
+		fprintf(ical,"$i dayrepeat %d $begin\n", a.repeatFrequency * 7);
 	    } else {
 		fprintf(ical,"$i weekdays ");
-		if(a.repeatOn & 1)
-		  fprintf(ical,"1 ");
-		if(a.repeatOn & 2)
-		  fprintf(ical,"2 ");
-		if(a.repeatOn & 4)
-		  fprintf(ical,"3 ");
-		if(a.repeatOn & 8)
-		  fprintf(ical,"4 ");
-		if(a.repeatOn & 16)
-		  fprintf(ical,"5 ");
-		if(a.repeatOn & 32)
-		  fprintf(ical,"6 ");
-		if(a.repeatOn & 64)
-		  fprintf(ical,"7 ");
+		for (i=0;i<7;i++)
+		  if (a.repeatDays[i])
+		    fprintf(ical,"%d ", i+1);
 		fprintf(ical,"\n");
 	    }
 	  } else if(a.repeatType == repeatYearly) {
-	    fprintf(ical,"$i monthrepeat %d $begin\n", 12 * a.repeatFreq);
+	    fprintf(ical,"$i monthrepeat %d $begin\n", 12 * a.repeatFrequency);
 	  }
 	  fprintf(ical,"$i start $begin\n");
 	  if (!a.repeatForever)
