@@ -118,19 +118,19 @@ int main(int argc, char *argv[])
 			a.repeatEnd.tm_mday, Month[a.repeatEnd.tm_mon], a.repeatEnd.tm_year+1900);
 	}
 
-	if(a.repeatFreq) {
+	if(a.repeatFrequency) {
 		if(a.repeatType == repeatDaily) {
 		        /* On the specified day... */
 			printf("REM %d %s %d ",a.begin.tm_mday,Month[a.begin.tm_mon],a.begin.tm_year+1900);
-			if(a.repeatFreq > 1) {
+			if(a.repeatFrequency > 1) {
 				/* And every x days afterwords */
-				printf("*%d ",a.repeatFreq);
+				printf("*%d ",a.repeatFrequency);
 			}
 		} else if(a.repeatType == repeatMonthlyByDate) {
 			/* On the x of every month */
 			printf("REM %d ", a.begin.tm_mday);
 
-			if(a.repeatFreq>1) {
+			if(a.repeatFrequency>1) {
 
 				/* if the month is equal to the starting month mod x */
 				sprintf(satisfy,"SATISFY \
@@ -140,10 +140,10 @@ int main(int argc, char *argv[])
 					a.begin.tm_year+1900,
 					a.begin.tm_mon+1,
 					a.begin.tm_mday,
-					a.repeatFreq,
+					a.repeatFrequency,
 					a.begin.tm_year+1900,
 					a.begin.tm_mon,
-					a.repeatFreq);
+					a.repeatFrequency);
 			} else {
 				sprintf(satisfy, "SATISFY [(trigdate()>=date(%d,%d,%d)) && (!isomitted(trigdate()))] ",
 					a.begin.tm_year+1900,
@@ -151,24 +151,14 @@ int main(int argc, char *argv[])
 					a.begin.tm_mday);
 			}
 		} else if(a.repeatType == repeatWeekly) {
+			int k;
 			/* On the chosen days of the week */
 			printf("REM ");
-			if(a.repeatOn & 1)
-				printf("Sun ");
-			if(a.repeatOn & 2)
-				printf("Mon ");
-			if(a.repeatOn & 4)
-				printf("Tue ");
-			if(a.repeatOn & 8)
-				printf("Wed ");
-			if(a.repeatOn & 16)
-				printf("Thu ");
-			if(a.repeatOn & 32)
-				printf("Fri ");
-			if(a.repeatOn & 64)
-				printf("Sat ");
+			for(k=0;k<7;k++)
+				if(a.repeatDays[i])
+					printf("%s ", Weekday[i]);
 
-			if(a.repeatFreq>1) {
+			if(a.repeatFrequency>1) {
 				/* if the week is equal to the starting week mod x */
 				sprintf(satisfy, "SATISFY \
 [(trigdate()>=date(%d,%d,%d)) &&\
@@ -177,11 +167,11 @@ int main(int argc, char *argv[])
 		a.begin.tm_year+1900,
 		a.begin.tm_mon+1,
 		a.begin.tm_mday,
-		a.repeatFreq,
+		a.repeatFrequency,
 		a.begin.tm_year+1900,
 		a.begin.tm_mon+1,
 		a.begin.tm_mday,
-		a.repeatFreq);
+		a.repeatFrequency);
 			} else {
 				sprintf(satisfy, "SATISFY [(trigdate()>=date(%d,%d,%d))  && (!isomitted(trigdate()))] ",
 					a.begin.tm_year+1900,
@@ -192,17 +182,17 @@ int main(int argc, char *argv[])
 			int day;
 			int weekday;
 			
-			if(a.repeatOn>=domLastSun) {
+			if(a.repeatDay>=domLastSun) {
 				day = 1;
-				weekday = a.repeatOn % 7;
+				weekday = a.repeatDay % 7;
 				printf("REM %s %d -7 ", Weekday[weekday], day);
 			} else {
-				day = a.repeatOn / 7 * 7 + 1;
-				weekday = a.repeatOn % 7;
+				day = a.repeatDay / 7 * 7 + 1;
+				weekday = a.repeatDay % 7;
 				printf("REM %s %d ", Weekday[weekday], day);
 			}
 
-			if( a.repeatFreq > 1) {
+			if( a.repeatFrequency > 1) {
 
 				sprintf(satisfy,"SATISFY \
 [(trigdate()>=date(%d,%d,%d)) && \
@@ -211,10 +201,10 @@ int main(int argc, char *argv[])
 					a.begin.tm_year+1900,
 					a.begin.tm_mon+1,
 					a.begin.tm_mday,
-					a.repeatFreq,
+					a.repeatFrequency,
 					a.begin.tm_year+1900,
 					a.begin.tm_mon,
-					a.repeatFreq);
+					a.repeatFrequency);
 			} else {
 				sprintf(satisfy, "SATISFY [(trigdate()>=date(%d,%d,%d))  && (!isomitted(trigdate()))] ",
 					a.begin.tm_year+1900,
@@ -225,7 +215,7 @@ int main(int argc, char *argv[])
 			/* On one day each year */
 			printf("REM %d %s ", a.begin.tm_mday, Month[a.begin.tm_mon]);
 	
-			if(a.repeatFreq>1) {
+			if(a.repeatFrequency>1) {
 
 				/* if the year is equal to the starting year, mod x */
 				sprintf(satisfy,"SATISFY \
@@ -235,9 +225,9 @@ int main(int argc, char *argv[])
 					a.begin.tm_year+1900,
 					a.begin.tm_mon+1,
 					a.begin.tm_mday,
-					a.repeatFreq,
+					a.repeatFrequency,
 					a.begin.tm_year+1900,
-					a.repeatFreq);
+					a.repeatFrequency);
 			} else {
 				sprintf(satisfy, "SATISFY [(trigdate()>=date(%d,%d,%d)) && (!isomitted(trigdate()))]",
 					a.begin.tm_year+1900,
